@@ -3,12 +3,13 @@ package JMX::Jmx4Perl::Nagios::CheckJmx4Perl;
 use strict;
 use warnings;
 use JMX::Jmx4Perl::Nagios::SingleCheck;
+use JMX::Jmx4Perl::Nagios::MessageHandler;
 use JMX::Jmx4Perl;
 use JMX::Jmx4Perl::Request;
 use JMX::Jmx4Perl::Response;
 use Data::Dumper;
 use Nagios::Plugin;
-use Nagios::Plugin::Functions qw(:codes %STATUS_TEXT);
+use Nagios::Plugin::Functions qw(:codes %ERRORS %STATUS_TEXT);
 use Time::HiRes qw(gettimeofday tv_interval);
 use Carp;
 use Text::ParseWords;
@@ -90,22 +91,38 @@ sub execute {
             push @requests,@{$check->get_requests($jmx,\@ARGV)};            
         }
         my $responses = $self->_send_requests($jmx,@requests);
+        #print Dumper($responses);
         my @extra_requests = ();
         my $nr_checks = scalar(@{$self->{checks}});
         if ($nr_checks == 1) {
-            my @r = $self->{checks}->[0]->extract_responses($responses,\@requests,{ target => $target_config });
-            push @extra_requests,@r if @r;
+            eval {
+                my @r = $self->{checks}->[0]->extract_responses($responses,\@requests,{ target => $target_config });
+                push @extra_requests,@r if @r;
+            };
+            $self->nagios_die($@) if $@;
         } else {
             my $i = 1;
             for my $check (@{$self->{checks}}) {
                 # A check can consume more than one response
-                my @r = $check->extract_responses($responses,\@requests,
-                                                    { 
-                                                     target => $target_config, 
-                                                     prefix => $self->_multi_check_prefix($check,$i++,$nr_checks),
-                                                     error_stat => $error_stat
-                                                    });
-                push @extra_requests,@r if @r;
+                my $prefix = $self->_multi_check_prefix($check,$i++,$nr_checks);
+                eval {
+                    my @r = $check->extract_responses($responses,\@requests,
+                                                        { 
+                                                         target => $target_config, 
+                                                         prefix => $prefix,
+                                                         error_stat => $error_stat
+                                                        });
+                    push @extra_requests,@r if @r;
+                };
+                if ($@) {
+                    my $txt = $@;
+                    $txt =~ s/^(.*?)\n.*$/$1/s;
+                    my $code = $np->opts->{'unknown-is-critical'} ? CRITICAL : UNKNOWN;
+                    $check->update_error_stats($error_stat,$code);
+                    $prefix =~ s/\%c/$STATUS_TEXT{$code}/g;
+                    my $msg_handler = $np->{msg_handler} || $np; 
+                    $msg_handler->add_message($code,$prefix . $txt);
+                }
             }
         }
         # Send extra requests, e.g. for switching on the history
@@ -135,12 +152,13 @@ output, which can be extracted from NagiosPlugin object.
 
 =cut 
 
-sub do_exit {
+sub do_exit {    
     my $self = shift;
     my $error_stat = shift;
     my $np = $self->{np};
 
-    my ($code,$message) = $np->check_messages(join => "\n", join_all => "\n");
+    my $msg_handler = $np->{msg_handler} || $np; 
+    my ($code,$message) = $msg_handler->check_messages(join => "\n", join_all => "\n");
     ($code,$message) = $self->_prepare_multicheck_message($np,$code,$message,$error_stat) if scalar(@{$self->{checks}}) > 1;
     
     $np->nagios_exit($code, $message);
@@ -154,43 +172,85 @@ sub _prepare_multicheck_message {
     my $error_stat = shift;
 
     my $summary;
+    my $labels = $self->{multi_check_labels} || {};
     my $nr_checks = scalar(@{$self->{checks}});
+    $code = $self->_check_for_UNKNOWN($error_stat,$code);
     if ($code eq OK) {
-        $summary = "All " . $nr_checks . " checks OK";
+        $summary = $self->_format_multicheck_ok_summary($labels->{summary_ok} ||
+                                                        "All %n checks OK",$nr_checks);
     } else {
-        my $nr_warnings = scalar(@{$np->messages->{warning} || []});
-        my $nr_errors = scalar(@{$np->messages->{critical} || []});
-        my @parts;
-        my $extra = "";
-        my $nr = 0;
-        for my $code (CRITICAL,WARNING,UNKNOWN) {
-            if (my $errs = $error_stat->{$code}) {
-                $extra .= scalar(@$errs) . " " . $STATUS_TEXT{$code} . " (" . join (",",@$errs) . "), ";
-                $nr += scalar(@$errs);
-            }
-        }
-        if ($nr > 0) {
-            # Cut off extra chars at the end
-            $extra = substr($extra,0,-2);
-        }
-        $summary = $nr . " of " . $nr_checks . " failed: " . $extra;
+        $summary = $self->_format_multicheck_failure_summary($labels->{summary_failure} ||
+                                                             "%e of %n checks failed [%d]",
+                                                             $nr_checks,
+                                                             $error_stat);
     }
     return ($code,$summary . "\n" . $message);
 }
 
+# UNKNOWN shadows everything else
+sub _check_for_UNKNOWN {
+    my $self = shift;
+    my $error_stat = shift;
+    my $code = shift;
+    return $error_stat->{UNKNOWN} && scalar(@$error_stat->{UNKNOWN}) ? UNKNOWN : $code;
+}
+
+sub _format_multicheck_ok_summary {
+    my $self = shift;
+    my $format = shift;
+    my $nr_checks = shift;
+    my $ret = $format;
+    $ret =~ s/\%n/$nr_checks/g;
+    return $ret;
+}
+
+sub _format_multicheck_failure_summary {
+    my $self = shift;
+    my $format = shift;
+    my $nr_checks = shift;
+    my $error_stat = shift;
+
+    my $ret = $format;
+
+    my $details = "";
+    my $total_errors = 0;
+    for my $code (UNKNOWN,CRITICAL,WARNING) {
+        if (my $errs = $error_stat->{$code}) {
+            $details .= scalar(@$errs) . " " . $STATUS_TEXT{$code} . " (" . join (",",@$errs) . "), ";
+            $total_errors += scalar(@$errs);
+        }
+    }
+    if ($total_errors > 0) {
+        # Cut off extra chars at the end
+        $details = substr($details,0,-2);
+    }
+    
+    $ret =~ s/\%d/$details/g;
+    $ret =~ s/\%e/$total_errors/g;
+    $ret =~ s/\%n/$nr_checks/g;
+    return $ret;
+}
+
 # Create a formatted prefix for multicheck output
 sub _multi_check_prefix {
     my $self = shift;
     my $check = shift;
     my $idx = shift;
     my $max = shift;
-    my $label = $check->{config}->{key} || $check->{config}->{name} || "";
+    
+    my $c = $check->{config};
+
     my $l = length($max);
+    
+    return sprintf("[%$l.${l}s] %%c ",$idx)
+      if (defined($c->{multicheckprefix}) && !length($c->{multicheckprefix}));
+    
+    my $label =  $c->{multicheckprefix} || $c->{name} || $c->{key} || "";
     return sprintf("[%$l.${l}s] %%c %s: ",$idx,$label);
 }
 
 
-# Sendp the requests via the build up agent
+# Send the requests via the build up agent
 sub _send_requests {
     my ($self,$jmx,@requests) = @_;
     my $o = $self->{opts};
@@ -307,7 +367,9 @@ sub _verify_and_initialize {
     my $config = $self->_get_config($o->config);
     # Now, if a specific check is given, extract it, too.
     my $check_configs;
+    #print Dumper($config);
     $check_configs = $self->_extract_checks($config,$o->check);
+    #print Dumper($check_configs);
     if ($check_configs) {
         for my $c (@$check_configs) {
             my $s_c = new JMX::Jmx4Perl::Nagios::SingleCheck($np,$c);
@@ -329,17 +391,9 @@ sub _verify_and_initialize {
         my $name = $check->name ? " [Check: " . $check->name . "]" : "";
         $self->nagios_die("An MBean name and a attribute/operation must be provided " . $name)
           if ((!$check->mbean || (!$check->attribute && !$check->operation)) && !$check->alias && !$check->value && !$check->script);
-        $self->verify_check($check,$name);
     }
 }
 
-sub verify_check {
-    my $self = shift;
-    my $check = shift;
-    my $name = shift;
-    my $np = $self->{np};
-}
-
 # Extract one or more check configurations which can be 
 # simple <Check>s or <MultiCheck>s
 sub _extract_checks {
@@ -355,6 +409,7 @@ sub _extract_checks {
         my $check_configs;
         unless ($config->{check}->{$check}) {
             $check_configs = $self->_resolve_multicheck($config,$check,$self->{cmd_args});
+            $self->_retrieve_mc_summary_label($config,$check);
         } else {
             my $check_config = $config->{check}->{$check};
             $check_configs = ref($check_config) eq "ARRAY" ? $check_config : [ $check_config ];
@@ -423,6 +478,24 @@ sub _resolve_multicheck {
     return $check_config;
 }
 
+sub _retrieve_mc_summary_label { 
+    my $self = shift;
+    my $config = shift;
+    my $check = shift;
+
+    my $multi_checks = $config->{multicheck};    
+    if ($multi_checks) { 
+        my $m_check = $multi_checks->{$check};
+        if ($m_check && ($m_check->{summaryok} || $m_check->{summaryfailure})) {
+            my $mc_labels = 
+            $self->{multi_check_labels} = {
+                                           summary_ok => $m_check->{summaryok},
+                                           summary_failure => $m_check->{summaryfailure}
+                                          };
+        }
+    }
+}
+
 sub _merge_multicheck_args {
     my $self = shift;
     my $check_params = shift;
@@ -535,8 +608,8 @@ EOP
         my $orig_val = '$' . $2;
         my $i = defined($3) ? $3 : $4;
         my $default = $5;
-        $default =~ s/^\s*(.*)+?\s*$/$1/ if $default; # Trim whitespace
         my $end = defined($6) ? $6 : "";
+        $default =~ s/^\s*(.*)+?\s*$/$1/ if $default; # Trim whitespace
         #print Dumper({start => $start, orig => $orig_val,end => $end, default=> $default, rest => $rest, i => $i}); 
         if (defined($args)) {
             my $repl = $args->[$i];            
@@ -581,7 +654,7 @@ EOP
 sub _parse_check_ref {
     my $self = shift;
     my $check_ref = shift;
-    if ($check_ref =~/^\s*([^(]+)\(([^)]*)\)\s*$/) {
+    if ($check_ref =~/^\s*(.+?)\((.*)\)\s*$/) {
         my $name = $1;
         my $args_s = $2;
         my $args = [ parse_line('\s*,\s*',0,$args_s) ];
@@ -631,6 +704,7 @@ sub create_nagios_plugin {
     $np->shortname(undef);
     $self->add_common_np_args($np);
     $self->add_nagios_np_args($np);
+    $np->{msg_handler} = new JMX::Jmx4Perl::Nagios::MessageHandler();
     $np->getopts();
     return $np;
 }
